use log::{LevelFilter, Log, Record, Metadata};
use std::fs::OpenOptions;
use std::io::{self, Write};
use std::path::PathBuf;
use chrono::Local;
use std::sync::RwLock;
use crate::error::{LogError, Result};

/// Configuration for logging setup
#[derive(Debug, Clone, PartialEq)]
pub struct LogConfig {
    pub log_file: Option<PathBuf>,
    pub log_level: LevelFilter,
    pub append_log: bool,
}

impl Default for LogConfig {
    fn default() -> Self {
        Self {
            log_file: None,
            log_level: LevelFilter::Info,
            append_log: false,
        }
    }
}

/// Custom writer that can write to either a file or stderr
#[derive(Debug)]
pub struct LogWriter {
    file: Option<std::fs::File>,
}

impl LogWriter {
    pub fn new(file: Option<std::fs::File>) -> Self {
        Self { file }
    }

    pub fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        match &mut self.file {
            Some(file) => file.write_all(buf),
            None => io::stderr().write_all(buf),
        }
    }

    pub fn flush(&mut self) -> io::Result<()> {
        match &mut self.file {
            Some(file) => file.flush(),
            None => io::stderr().flush(),
        }
    }
}

impl Write for LogWriter {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.write_all(buf)?;
        Ok(buf.len())
    }

    fn flush(&mut self) -> io::Result<()> {
        self.flush()
    }
}

#[derive(Debug)]
struct SimpleLogger {
    writer: RwLock<LogWriter>,
    level: LevelFilter,
}

impl SimpleLogger {
    fn new(writer: LogWriter, level: LevelFilter) -> Self {
        Self { 
            writer: RwLock::new(writer),
            level,
        }
    }
}

impl Log for SimpleLogger {
    fn enabled(&self, metadata: &Metadata) -> bool {
        metadata.level() <= self.level
    }

    fn log(&self, record: &Record) {
        if !self.enabled(record.metadata()) {
            return;
        }

        let message = format!(
            "{} [{:<5}] - {}\n",
            Local::now().format("%Y-%m-%d %H:%M:%S%.3f"),
            record.level(),
            record.args()
        );

        if let Ok(mut writer) = self.writer.write() {
            if let Err(e) = writer.write_all(message.as_bytes()) {
                eprintln!("Failed to write log message: {}", e);
            }
        } else {
            eprintln!("Failed to acquire write lock for logging");
        }
    }

    fn flush(&self) {
        if let Ok(mut writer) = self.writer.write() {
            if let Err(e) = writer.flush() {
                eprintln!("Failed to flush logger: {}", e);
            }
        } else {
            eprintln!("Failed to acquire write lock for flushing");
        }
    }
}

/// Initialize logging for production use
///
/// # Errors
///
/// Returns an error if:
/// - Failed to create log directory
/// - Failed to open log file
/// - Failed to write initial log header
/// - Failed to set global logger
/// - Logger is already initialized
pub fn init_prod_logging(config: LogConfig) -> Result<()> {
    let log_file = if let Some(log_path) = &config.log_file {
        // Create parent directory if it doesn't exist
        if let Some(parent) = log_path.parent() {
            std::fs::create_dir_all(parent).map_err(LogError::Io)?;
        }

        let file = OpenOptions::new()
            .create(true)
            .write(true)
            .append(config.append_log)
            .truncate(!config.append_log)
            .open(log_path)
            .map_err(LogError::Io)?;

        // Write header for new log files
        if !config.append_log {
            writeln!(
                &file,
                "=== Log started at {} ===",
                Local::now().format("%Y-%m-%d %H:%M:%S")
            ).map_err(LogError::Io)?;
        }
        Some(file)
    } else {
        None
    };

    let writer = LogWriter::new(log_file);
    let logger = SimpleLogger::new(writer, config.log_level);

    // Set the logger
    log::set_boxed_logger(Box::new(logger))
        .map_err(|e| LogError::Init(e.to_string()))?;

    log::set_max_level(config.log_level);
    Ok(())
} 